;+
; NAME:
;       IAA_PLOT
;
; PURPOSE:
;
;       This program is a simple wrapper for the IDL PLOT command. The
;       main purpose of the program is to demonstrate one way the
;       IAA_PLOTCONFIG program can be used to update plot parameters.
;
; AUTHOR:
;
;       David Fanning, Ph.D.
;
;       Modified by Jos Enrique Ruiz
;       December 2006
;
; CATEGORY:
;
;       Graphics
;
; CALLING SEQUENCE:
;
;       IAA_Plot, x, y
;       IAA_Plot, xx, yy, /Overplot
;
; INPUT PARAMETERS:
;
;       x - The independent data. If y is not present, x is taken to be the dependent data.
;
;       y - The dependent data. The vectors x and y must be the same length.
;
;       xs - The independent data to overplot. If yy is not present, xx is taken to be the dependent data.
;
;       yy - The dependent data to overplot. The vectors xx and yy must be the same length.
;
; INPUT KEYWORDS:
;
;       BACKGROUND - The name of a background color. (See below for a list of color names.)
;         By default on 24-bit systems: 'IVORY'. Uses 'GRAY' on 8-bit systems.
;
;       COLOR - The name of the plot color. (See below for a list of color names.)
;         By default on 24-bit systems: 'SADDLE BROWN'. Uses 'GREEN' on 8-bit systems.
;         When OVERPLOTing, use the COLOR keyword to specify the color of the overplot.
;
;       DATACOLOR - The name of the data color. By default the same as the COLOR keyword.
;         (See below for a list of color names.) When OVERPLOTing, use the COLOR keyword
;         to specify the color of the overplot.
;
;       OVERPLOT - Set this keyword to overplot data into the IAA_PLOT window.
;         If multiple windows are on the display, select the one to overplot into
;         by selecting it with the cursor.
;
;       PSYM - The plot symbol value. By default, 18. Possible values are:
;         0 - Dot
;         1 - Filled Circle
;         2 - Filled Upward Triangle
;         3 - Filled Downward Triangle
;         4 - Filled Diamond
;         5 - Filled Square
;         6 - Open Circle
;         7 - Open Upward Triangle
;         8 - Open Downward Triangle
;         9 - Open Diamond
;        10 - Open Square
;        11 - Plus Sign
;        12 - X
;        13 - Star
;        14 - Filed Rightfacing Triangle
;        15 - Filled Leftfacing Triangle
;        16 - Open Rightfacing Triangle
;        17 - Open Leftfacing Triangle
;        18 - No Symbol (the default).
;
;       TITLE - The title of the plot. By default, a null string.
;
;       XLOG = Set this keyword to use logarithmic axis styling on the X axis.
;
;       XTITLE - The title of the X axis of the plot. By default, a null string.
;
;       YLOG = Set this keyword to use logarithmic axis styling on the Y axis.
;
;       YTITLE - The title of the Y axis of the plot. By default, a null string.
;
;       In addition, any keyword appropriate for the IAA_PLOTCONFIG object program can be used.
;       Among those keywords, are these most popular ones:
;
;       CHARSIZE - The character size of the plot. By default, 1.0.
;
;       CHARTHICK - The character thickness of the plot. By default, 1.0.
;
;       FONT - The type of plot font: -1=Hershey, 0=Hardware, 1=True-Type. By default, !P.FONT.
;
;       LINESTYLE - The plot linestyle. By default, 0. Possible values are:
;         0 - Solid Line
;         1 - Dotted
;         2 - Dashed
;         3 - Dash Dot
;         4 - Dash Dot Dot
;         5 - Long Dash
;         6 - No Line
;
;       POSITION - The position of the plot in the plot window in normalized coordinates. By default, [0.20, 0.15, 0.95, 0.95].
;
;       SYMSIZE - The plot symbol size. By default, 1.0.
;
;       THICK - The plot line thickness. By default, 1.0.
;
;       TICKLEN - The plot tick length. By default, 0.02.
;
; COLOR NAMES:
;
;        The following color names can be used for BACKGROUND, COLOR and DATACOLOR keywords:
;
;           White, Snow, Ivory, Light Yellow, Cornsilk, Beige, Seashell, Linen, Antique White,
;           Papaya, Almond, Bisque, Moccasin, Wheat, Burlywood, Tan, Light Gray, Lavender,
;           Medium Gray, Gray, Slate Gray, Dark Gray , Charcoal, Black, Light Cyan, Powder Blue,
;           Sky Blue, Steel Blue, Dodger Blue, Royal Blue, Blue, Navy, Honeydew, Pale Green,
;           Aquamarine, Spring Green, Cyan, Turquoise, Sea Green, Forest Green, Green Yellow,
;           Chartreuse, Lawn Green, Green, Lime Green, Olive Drab, Olive, Dark Green, Pale Goldenrod,
;           Khaki, Dark Khaki, Yellow, Gold, Goldenrod, Dark Goldenrod, Saddle Brown, Rose,
;           Pink, Rosy Brown, Sandy Brown, Peru, Indian Red, Chocolate, Sienna, Dark Salmon,
;           Salmon, Light Salmon, Orange, Coral, Light Coral, Firebrick, Brown, Hot Pink,
;           Deep Pink, Magenta, Tomato, Orange Red, Red, Violet Red, Maroon, Thistle, Plum,
;           Violet, Orchid, Medium Orchid, Dark Orchid, Blue Violet, and Purple.
;
;
; COMMON BLOCK:
;
;       The addition of the OVERPLOT keyword required a COMMON block named IAA_PLOT_COMMMON
;       to store the program information pointer. This pointer is loaded in the COMMON block
;       when the keyboard focus changes. Thus, to overplot into an IAA_PLOT window, first
;       select the window with the cursor.
;
; RESTRICTIONS
;
;       Colors will be loaded in the color table.
;
; EXAMPLE:
;
;       x = Findgen(11) & y = Findgen(11)
;       IAA_PLOT, x, y
;       IAA_PLOT, Reverse(x), y, /Overplot, Linestyle=2
;


PRO IAA_Overplot__Define

   struct = { IAA_OVERPLOT, $      ; The overplot structure.
              dep: Ptr_New(), $    ; The dependent data.
              indep: Ptr_New(), $  ; The independent data.
              extra: Ptr_New(), $  ; Any extra keywords.
              color: '', $         ; The name of the overplot color.
              thick:0., $          ; The thickness of the overplot line.
              linestyle:0L, $      ; The linestyle for overplotting.
              symsize:0., $        ; The symbol size overplotting.
              psym:0L, $           ; The symbol used for overplotting.
              noline:0L $          ; A flag to tell if symbols should be conntect by lines (-1) on not (1).
             }
END; -------------------------------------------------------------------------

PRO IAA_Plot_about, event

value = [$
'', $
'IAA Plot Version 1.0', $
'', $
'Jos Enrique Ruiz', $
'Dept. del Sistema Solar', $
'Instituto Astrofsica de Andaluca', $
'CSIC', $
'' $
]

about = DIALOG_MESSAGE(value, /information)

END ;---------------------------------------------------------------------------

PRO IAA_Plot_Clear_Overplots, event

; This procedure clears any overplots from the program.

   ; Get info structure.

Widget_Control, event.top, Get_UValue=infoptr

   ; Free the overplot pointer.

IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
   FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      Ptr_Free, struct.dep
      Ptr_Free, struct.indep
      Ptr_Free, struct.extra
      ENDFOR
   Ptr_Free, (*infoptr).oplotinfo
ENDIF ELSE Ptr_Free, (*infoptr).oplotinfo

Ptr_Free, (*infoptr).OColorArr
Ptr_Free, (*infoptr).OLineStyleArr
Ptr_Free, (*infoptr).OThickArr
Ptr_Free, (*infoptr).OSymSizeArr
Ptr_Free, (*infoptr).OPSymArr
Ptr_Free, (*infoptr).ONoLineArr

(*infoptr).nboverplots=0
(*infoptr).nbplotsloaded=0


   ; Turn overplot button off.

Widget_Control, (*infoptr).clearoplotID, Sensitive=0
Widget_Control, (*infoptr).updoplotID, Sensitive=0

(*infoptr).overplotconfig->Destroy

   ; Redraw the plot.

IAA_Plot_DrawThePlot, infoptr


END; -------------------------------------------------------------------------


PRO IAA_Plot_DrawThePlot, infoptr, Overplot=overplot

; This procedure draws the plot and overplot on the
; output device.

   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /cancel
   ok = Error_Message(/Traceback)
ENDIF

   ; Set current graphics window for devices that support it.

IF (!D.Flags AND 256) NE 0 THEN WSet, (*infoptr).wid

   ; Restore plotting and axis system variables. Required for overplotting.

!X = (*infoptr).x
!Y = (*infoptr).y
!P = (*infoptr).p

   ; Draw the plot.

plotkeywords = (*infoptr).plotconfig->GetKeywords()

IF Keyword_Set(overplot) EQ 0 THEN BEGIN
   IF (!D.Flags AND 256) EQ 0 THEN BEGIN
      PolyFill, [0, 0, 1, 1, 0], [0, 1, 1, 0, 0], /Normal, Color=plotkeywords.background
      Plot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords, /NoData, /NoErase
   ENDIF ELSE BEGIN
      Plot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords, /NoData
   ENDELSE
   plotkeywords.color = plotkeywords.datacolor
   OPlot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords
ENDIF

   ; Draw any overplots.

IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
   FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      symbol = (*infoptr).overplotconfig->Symbol(abs(struct.psym))*struct.noline
      OPlot, *struct.indep, *struct.dep, _Extra=*struct.extra, $
         Color=FSC_Color(struct.color, !D.Table_Size-4+j), $
         PSym=symbol, LineStyle=struct.linestyle, $
         SymSize=struct.symsize, Thick=struct.thick
    ENDFOR
ENDIF

   ; Draw errplots

IF (N_Elements((*infoptr).xerrplot) GT 1) THEN BEGIN
   oploterr, (*infoptr).indep, (*infoptr).dep, (*infoptr).xerrplot, (*infoptr).yerrplot
ENDIF

IF (N_Elements((*infoptr).yerrplot) GT 1) THEN BEGIN
   oploterr, (*infoptr).indep, (*infoptr).dep, (*infoptr).yerrplot
ENDIF

   ; Update the system variables.

(*infoptr).x = !X
(*infoptr).y = !Y
(*infoptr).p = !P

END; -------------------------------------------------------------------------


PRO IAA_Plot_PostScript, event

; This event handler creates PostScript output.

   ; Get info structure.

Widget_Control, event.top, Get_UValue=infoptr

   ; Make the display window current.

WSet, (*infoptr).wid

keywords = PSWindow()
pskeywords = PSConfig(Cancel=cancelled, _Extra=keywords, Group_Leader=event.top)
IF cancelled THEN RETURN

   ; Configure the PostScript device.

thisDevice = !D.Name
Set_Plot, 'PS'
Device, _Extra=pskeywords

   ; Display the grapics. Background color will have to be done
   ; separately. And if color is turned off, we have have to do
   ; things differntly, too.

plotkeywords = (*infoptr).plotconfig->GetKeywords()

IF pskeywords.color EQ 1 THEN BEGIN

   PolyFill, [0, 0, 1, 1, 0], [0, 1, 1, 0, 0], /Normal, Color=plotkeywords.background
   Plot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords, /NoData, /NoErase
   plotkeywords.color = plotkeywords.datacolor
   OPlot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords

ENDIF ELSE BEGIN

   plotkeywords.color = !P.Color
   plotkeywords.datacolor = !P.Color
   plotkeywords.background = !P.Background
   Plot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords, /NoData
   OPlot, (*infoptr).indep, (*infoptr).dep, _Extra=plotkeywords

ENDELSE


   ; Any overplotting to do?

IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
   FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      symbol = (*infoptr).overplotconfig->Symbol(abs(struct.psym))*struct.noline
      OPlot, *struct.indep, *struct.dep, _Extra=*struct.extra, $
         Color=FSC_Color(struct.color, !D.Table_Size-4+j), $
         PSym=symbol, LineStyle=struct.linestyle, $
         Symsize=struct.symsize, Thick=struct.thick
   ENDFOR
ENDIF


   ; Draw errplots

IF (N_Elements((*infoptr).xerrplot) GT 1) THEN BEGIN
   oploterr, (*infoptr).indep, (*infoptr).dep, (*infoptr).xerrplot, (*infoptr).yerrplot
ENDIF

IF (N_Elements((*infoptr).yerrplot) GT 1) THEN BEGIN
   oploterr, (*infoptr).indep, (*infoptr).dep, (*infoptr).yerrplot
ENDIF

   ; Close the PostScript file and clean up.

Device, /Close_File
Set_Plot, thisDevice

END; -------------------------------------------------------------------------

PRO IAA_Plot_Print, event

; This event handler executes the command in the Printer device.

   ; Set up the printer.

ok = Dialog_PrinterSetup()
IF NOT ok THEN RETURN

   ; Get info structure and printer orientation.

Widget_Control, event.top, Get_UValue=infoptr
Widget_Control, event.id, Get_UValue=orientation

   ; Save the current graphics device.

thisDevice = !D.Name

   ; Make the display window current.

WSet, (*infoptr).wid

   ; With printers, you have to load your drawing colors
   ; *before* you enter the PRINTER device and then copy
   ; them into the device. It's a pain in the ol' wazoo,
   ; but there is no help for it. :-(

plotkeywords = (*infoptr).plotconfig->GetKeywords()
TVLCT, FSC_Color(plotkeywords.name_background, /Triple), !P.Background
TVLCT, FSC_Color(plotkeywords.name_color, /Triple), !D.Table_Size-2
TVLCT, FSC_Color(plotkeywords.name_datacolor, /Triple), !D.Table_Size-3
IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
   FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      TVLCT, FSC_Color(struct.color, /Triple),!D.Table_Size-4+j
   ENDFOR
ENDIF

   ; Set up the printer. You may have to adjust the fudge factors
   ; to account for the printable area offset.

CASE orientation OF
   'PORTRAIT': BEGIN
      keywords = PSWindow(/Printer, Fudge=0.25)
      Set_Plot, 'PRINTER', /Copy
      Device, Portrait=1
      ENDCASE
   'LANDSCAPE': BEGIN
      keywords = PSWindow(/Printer, /Landscape, Fudge=0.25)
      Set_Plot, 'PRINTER', /Copy
      Device, Landscape=1
      ENDCASE
ENDCASE
Device, _Extra=keywords

   ; Display the graphics.

IAA_Plot_DrawThePlot, infoptr

   ; Close the printer file and clean up.

Device, /Close_Document
Set_Plot, thisDevice

END ;----------------------------------------------------------------------------------------

PRO IAA_Plot_SaveAs, event

; Saves the current display window as output files.

   ; Get the info structure and the appropriate file extension.

Widget_Control, event.top, Get_UValue=infoptr
Widget_Control, event.id, Get_UValue=file_extension

   ; Base name for file output.

basename = 'iaa_plot'

   ; Take a snapshot of the display window and write the file.

WSet, (*infoptr).wid
CASE file_extension OF
   'BMP'  : image = TVREAD(Filename = basename, /BMP)
   'GIF'  : image = TVREAD(Filename = basename, /GIF)
   'PICT' : image = TVREAD(Filename = basename, /PICT)
   'JPG'  : image = TVREAD(Filename = basename, /JPEG)
   'TIF'  : image = TVREAD(Filename = basename, /TIFF)
   'PNG'  : image = TVREAD(Filename = basename, /PNG)
ENDCASE

END ;----------------------------------------------------------------------------------------



PRO IAA_Plot_Quit, event
Widget_Control, event.top, /Destroy
END ;-----------------------------------------------------------------------------

PRO IAA_Plot_Cleanup, tlb

   ; This is the cleanup routine for the program. All
   ; created objects and pointers must be destroyed.

Widget_Control, tlb, Get_UValue=infoptr

Obj_Destroy, (*infoptr).xaxis
Obj_Destroy, (*infoptr).yaxis
Obj_Destroy, (*infoptr).plotconfig
Obj_Destroy, (*infoptr).overplotconfig


IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
   FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      Ptr_Free, struct.dep
      Ptr_Free, struct.indep
      Ptr_Free, struct.extra
   ENDFOR
   Ptr_Free, (*infoptr).oplotinfo
ENDIF ELSE Ptr_Free, (*infoptr).oplotinfo

Ptr_Free, (*infoptr).OColorArr
Ptr_Free, (*infoptr).OLineStyleArr
Ptr_Free, (*infoptr).OThickArr
Ptr_Free, (*infoptr).OSymSizeArr
Ptr_Free, (*infoptr).OPSymArr
Ptr_Free, (*infoptr).ONoLineArr

Ptr_Free, infoptr

END ;-----------------------------------------------------------------------------

PRO IAA_Plot_Configure_Button, event

   ; This event handler responds to requests to configure the
   ; plot parameters. Events will be sent to the buttonbase widget.

Widget_Control, event.top, Get_UValue=infoptr

   ; Call the GUI method on the Plot Configuration object.

Widget_Control, event.top, Update=0
(*infoptr).plotconfig->GUI, Group_Leader=event.top, All_Events=1, $
   NotifyID=[(*infoptr).buttonbase, event.top]

Widget_Control, event.top, Update=1

END ;-----------------------------------------------------------------------------

PRO IAA_Plot_Overplots_Button, event

   ; This event handler responds to requests to configure the
   ; overplot parameters. Events will be sent to the buttonbase widget.

Widget_Control, event.top, Get_UValue=infoptr

   ; Call the GUI method on the Plot Configuration object.

Widget_Control, event.top, Update=0
(*infoptr).overplotconfig->GUI, Group_Leader=event.top, $
   NotifyID=[(*infoptr).buttonbase, event.top], OverplotArr=(*infoptr).oplotinfo
Widget_Control, event.top, Update=1

END ;-----------------------------------------------------------------------------

PRO IAA_Plot_Load, event

   ; This event handler responds to requests to load plot settings.

    Widget_Control, event.top, Get_UValue=infoptr

    filename = DIALOG_PICKFILE(/READ, FILE = '*.sav')

    plotkeywords = (*infoptr).plotconfig->GetKeywords()
    TitleNow = plotkeywords.TITLE
    XTitleNow = plotkeywords.XTITLE
    YTitleNow = plotkeywords.YTITLE


   ; Restore Plot Settings

    IF filename NE '' THEN BEGIN

        restore, filename

        IF TitleNow EQ '' THEN BEGIN
        (*infoptr).plotconfig->SetProperty, Title=plotkeywords.TITLE
        ENDIF ELSE BEGIN
        (*infoptr).plotconfig->SetProperty, Title=TitleNow
        ENDELSE

        (*infoptr).plotconfig->SetProperty, Background=plotkeywords.NAME_BACKGROUND
        (*infoptr).plotconfig->SetProperty, Charsize=plotkeywords.CHARSIZE
        (*infoptr).plotconfig->SetProperty, Charthick=plotkeywords.CHARTHICK
        (*infoptr).plotconfig->SetProperty, Color=plotkeywords.NAME_COLOR
        (*infoptr).plotconfig->SetProperty, Datacolor=plotkeywords.NAME_DATACOLOR
        (*infoptr).plotconfig->SetProperty, Font=plotkeywords.FONT
        (*infoptr).plotconfig->SetProperty, Linestyle=plotkeywords.LINESTYLE
        (*infoptr).plotconfig->SetProperty, Position=plotkeywords.POSITION
        (*infoptr).plotconfig->SetProperty, PSym=plotkeywords.SYMBOL
        (*infoptr).plotconfig->SetProperty, Symsize=plotkeywords.SYMSIZE
        (*infoptr).plotconfig->SetProperty, Subtitle=plotkeywords.SUBTITLE
        (*infoptr).plotconfig->SetProperty, Thick=plotkeywords.THICK
        (*infoptr).plotconfig->SetProperty, Ticklen=plotkeywords.TICKLEN

        IF XTitleNow EQ '' THEN BEGIN
        (*infoptr).xaxis->SetProperty, Title=plotkeywords.XTITLE, /xaxis
        ENDIF ELSE BEGIN
        (*infoptr).xaxis->SetProperty, Title=XTitleNow
        ENDELSE

        (*infoptr).xaxis->SetProperty, Charsize=plotkeywords.XCHARSIZE, /xaxis
        (*infoptr).xaxis->SetProperty, Gridstyle=plotkeywords.XGRIDSTYLE, /xaxis
        (*infoptr).xaxis->SetProperty, Minor=plotkeywords.XMINOR, /xaxis
        (*infoptr).xaxis->SetProperty, Style=plotkeywords.XSTYLE, /xaxis
        (*infoptr).xaxis->SetProperty, Thick=plotkeywords.XTHICK, /xaxis
        (*infoptr).xaxis->SetProperty, Tickformat=plotkeywords.XTICKFORMAT, /xaxis
        (*infoptr).xaxis->SetProperty, Ticklen=plotkeywords.XTICKLEN, /xaxis
        (*infoptr).xaxis->SetProperty, Ticks=plotkeywords.XTICKS, /xaxis
        (*infoptr).xaxis->SetProperty, Log=plotkeywords.XLOG, /xaxis
        ;(*infoptr).xaxis->SetProperty, Range=plotkeywords.XRANGE, /xaxis

        IF YTitleNow EQ '' THEN BEGIN
        (*infoptr).yaxis->SetProperty, Title=plotkeywords.YTITLE, /yaxis
        ENDIF ELSE BEGIN
        (*infoptr).yaxis->SetProperty, Title=YTitleNow
        ENDELSE

        (*infoptr).yaxis->SetProperty, Charsize=plotkeywords.YCHARSIZE, /yaxis
        (*infoptr).yaxis->SetProperty, Gridstyle=plotkeywords.YGRIDSTYLE, /yaxis
        (*infoptr).yaxis->SetProperty, Minor=plotkeywords.YMINOR, /yaxis
        (*infoptr).yaxis->SetProperty, Style=plotkeywords.YSTYLE, /yaxis
        (*infoptr).yaxis->SetProperty, Thick=plotkeywords.YTHICK, /yaxis
        (*infoptr).yaxis->SetProperty, Tickformat=plotkeywords.YTICKFORMAT, /yaxis
        (*infoptr).yaxis->SetProperty, Ticklen=plotkeywords.YTICKLEN, /yaxis
        (*infoptr).yaxis->SetProperty, Ticks=plotkeywords.YTICKS, /yaxis
        (*infoptr).yaxis->SetProperty, Log=plotkeywords.YLOG, /yaxis
        ;(*infoptr).yaxis->SetProperty, Range=plotkeywords.YRANGE, /yaxis

        IF N_Elements(OColorArr) GT 0 THEN  (*infoptr).OColorArr=Ptr_New(OColorArr)
        IF N_Elements(OThickArr) GT 0 THEN  (*infoptr).OThickArr=Ptr_New(OThickArr)
        IF N_Elements(OLineStyleArr) GT 0 THEN  (*infoptr).OLineStyleArr=Ptr_New(OLineStyleArr)
        IF N_Elements(OSymSizeArr) GT 0 THEN  (*infoptr).OSymSizeArr=Ptr_New(OSymSizeArr)
        IF N_Elements(OPSymArr) GT 0 THEN  (*infoptr).OPSymArr=Ptr_New(OPSymArr)
        IF N_Elements(ONoLineArr) GT 0 THEN  (*infoptr).ONoLineArr=Ptr_New(ONoLineArr)
        IF N_Elements(OColorArr) GT 0 THEN  (*infoptr).nbplotsloaded=N_Elements(OColorArr)

        IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN
         IF N_Elements(OColorArr) GE N_Elements(*(*infoptr).oplotinfo) THEN $
         limit = N_Elements(*(*infoptr).oplotinfo) ELSE $
         limit = N_Elements(OColorArr)

            FOR j=0,limit-1 DO BEGIN
                 (*(*infoptr).oplotinfo)[j].color=OColorArr[j]
                 (*(*infoptr).oplotinfo)[j].thick=OThickArr[j]
                 (*(*infoptr).oplotinfo)[j].linestyle=OLineStyleArr[j]
                 (*(*infoptr).oplotinfo)[j].symsize=OSymSizeArr[j]
                 (*(*infoptr).oplotinfo)[j].psym=OPSymArr[j]
                 (*(*infoptr).oplotinfo)[j].noline=ONoLineArr[j]
            ENDFOR
         ENDIF


   ; Display the graphics.

    IAA_Plot_DrawThePlot, infoptr

    ENDIF

END ;-----------------------------------------------------------------------------


PRO IAA_Plot_Save_Settings, event

   ; This event handler responds to requests to save plot settings.

Widget_Control, event.top, Get_UValue=infoptr

plotkeywords = (*infoptr).plotconfig->GetKeywords()

filename = DIALOG_PICKFILE(/WRITE, FILE = 'plotsettings.sav')

IF Ptr_Valid((*infoptr).oplotinfo) THEN BEGIN

    OColorArr = StrArr((*infoptr).nboverplots)
    OThickArr = IntArr((*infoptr).nboverplots)
    OLineStyleArr = IntArr((*infoptr).nboverplots)
    OSymSizeArr = IntArr((*infoptr).nboverplots)
    OPSymArr = IntArr((*infoptr).nboverplots)
    ONoLineArr = IntArr((*infoptr).nboverplots)

    FOR j=0,N_Elements(*(*infoptr).oplotinfo)-1 DO BEGIN
      struct = (*(*infoptr).oplotinfo)[j]
      OColorArr[j]=struct.color
      OThickArr[j]=struct.thick
      OLineStyleArr[j]=struct.linestyle
      OSymSizeArr[j]=struct.symsize
      OPSymArr[j]=struct.psym
      ONoLineArr[j]=struct.noline
    ENDFOR

    SAVE, FILENAME = filename, plotkeywords, OColorArr, OThickArr, OLineStyleArr, OSymSizeArr, OPSymArr, ONoLineArr

ENDIF ELSE BEGIN

    SAVE, FILENAME = filename, plotkeywords

ENDELSE


END ;-----------------------------------------------------------------------------

PRO IAA_Plot_Motion, event

   ; This event handler responds to mouse motion events

Widget_Control, event.top, Get_UValue=infoptr

WSET, (*infoptr).wid

data = CONVERT_COORD(event.x, event.y, /DEVICE, /TO_DATA)

label = 'X: '+string(data[0])+'   Y: '+string(data[1])
widget_control, (*infoptr).labelID, set_value=label


END ;-----------------------------------------------------------------------------


PRO IAA_Plot_Resize, event

thisEvent = Tag_Names(event, /Structure_Name)

IF thisEvent EQ 'WIDGET_BASE' THEN BEGIN

      ; This event handler responds to TLB re-size events.

   Widget_Control, event.top, Get_UValue=infoptr

      ; Make the display window the current graphics window.

   WSet, (*infoptr).wid

      ; Resize the draw widget.

   Widget_Control, (*infoptr).drawID, Draw_XSize=(*infoptr).xminsize > event.x, $
      Draw_YSize=(*infoptr).yminsize > event.y

      ; Display the graphics.

   IAA_Plot_DrawThePlot, infoptr
   RETURN
ENDIF


IF thisEvent EQ 'WIDGET_KBRD_FOCUS' THEN BEGIN

   ; This event handler responds to TLB keyboard focus events.

   IF event.enter EQ 0 THEN RETURN

      ; Declare the common block with the info pointer.

   COMMON IAA_PLOT_COMMON, thePointer

   Widget_Control, event.top, Get_UValue=infoptr

      ; Make the display window the current graphics window.

   WSet, (*infoptr).wid

      ; Set the pointer to the common block.

   thePointer = infoptr
   RETURN
ENDIF

END ;-----------------------------------------------------------------------------



PRO IAA_Plot_Configuration_Events, event

   ; This event handler responds to events comimg from
   ; the Plot Configuration widget via the buttonbase widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /cancel
   ok = Error_Message(/Traceback)
ENDIF
Widget_Control, event.top, Get_UValue=infoptr

IF event.NAME EQ 'OVERPLOT' THEN BEGIN

    stroplot = event.Object->GetKeywords()
    index = stroplot.oplotind

    (*(*infoptr).oplotinfo)[index].color=stroplot.color
    (*(*infoptr).oplotinfo)[index].thick=stroplot.thick
    (*(*infoptr).oplotinfo)[index].linestyle=stroplot.linestyle
    (*(*infoptr).oplotinfo)[index].symsize=stroplot.symsize
    (*(*infoptr).oplotinfo)[index].psym=stroplot.psym
    (*(*infoptr).oplotinfo)[index].noline=stroplot.noline

ENDIF

   ; Display the graphics.


IAA_Plot_DrawThePlot, infoptr

END ;-----------------------------------------------------------------------------



PRO IAA_Plot, $
x, $
y, $
XTitle=xtitle, $
YTitle=ytitle, $
Title=title, $
Overplot=overplot, $
Color=color, $
DataColor=datacolor, $
PSym=psym, $
XLog=xlog, $
YLog=ylog, $
Group_Leader=group_leader, $
Xerrplot=xerrplot, $
Yerrplot=yerrplot, $
_Extra=extra

   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   ok = Error_Message(/Traceback)
   RETURN
ENDIF

   ; Common block required for overplotting capability.

COMMON IAA_PLOT_COMMON, infoPtr

   ; Check color keywords. Differences for display depth
   ; and overplotting verses plotting.

Device, Get_Visual_Depth=theDepth
IF theDepth GT 8 THEN BEGIN
   IF Keyword_Set(overplot) THEN BEGIN
      IF N_Elements(color) EQ 0 THEN BEGIN
         IF Ptr_Valid(infoptr) THEN BEGIN
            plotkeywords = (*infoptr).plotconfig->GetKeywords()
            color = plotkeywords.name_datacolor
         ENDIF ELSE color = 'SADDLE BROWN'
       ENDIF
   ENDIF ELSE BEGIN
      IF N_Elements(background) EQ 0 THEN background = 'IVORY'
      IF N_Elements(color) EQ 0 THEN color = 'SADDLE BROWN'
      IF N_Elements(datacolor) EQ 0 THEN datacolor = color
   ENDELSE
ENDIF ELSE BEGIN
   IF Keyword_Set(overplot) THEN BEGIN
      IF N_Elements(color) EQ 0 THEN BEGIN
         IF Ptr_Valid(infoptr) THEN BEGIN
            plotkeywords = (*infoptr).plotconfig->GetKeywords()
            color = plotkeywords.name_datacolor
         ENDIF ELSE color = 'GREEN'
       ENDIF
   ENDIF ELSE BEGIN
      IF N_Elements(background) EQ 0 THEN background = 'GRAY'
      IF N_Elements(color) EQ 0 THEN color = 'GREEN'
      IF N_Elements(datacolor) EQ 0 THEN datacolor = color
   ENDELSE
ENDELSE

   ; Create overplot configuration object.
IF Ptr_Valid(infoptr) THEN BEGIN
    IF Obj_Valid((*infoptr).overplotconfig) EQ 0 THEN BEGIN
    overplotconfig=Obj_New('IAA_OverPlotConfig')
    ENDIF ELSE BEGIN
    overplotconfig=(*infoptr).overplotconfig
    ENDELSE
ENDIF ELSE BEGIN
    overplotconfig=Obj_New('IAA_OverPlotConfig')
ENDELSE


   ; Make sure color names are strings.

IF N_Elements(psym) EQ 0 THEN psym = 18 ; No symbol.

   ; Are you overplotting something?

IF Keyword_Set(overplot) EQ 0 THEN BEGIN

      ; Do we have the color right?

   IF Size(color, /TName) NE 'STRING' THEN Message, 'COLOR must be specified as color NAME.', /NoName

   nboverplots = 0
   nbplotsloaded = 0
   infoPtr = Ptr_New()

ENDIF ELSE BEGIN

      ; Is there a current plot?

   IF Ptr_Valid(infoptr) EQ 0 THEN BEGIN
      ok = Dialog_Message('There is no current plot to overplot onto.')
      RETURN
   ENDIF

      ; Because of common block, only one IAA_PLOT at a time allowed.

   IF XRegistered('iaa_plot') GT 1 THEN BEGIN
      ok = Dialog_Message('Only one IAA_PLOT can be used at once if overplotting.')
      RETURN
   ENDIF

      ; Sort out the positional parameters.

   IF N_Elements(x) EQ 0 THEN BEGIN
      ok = Dialog_Message('Please supply data for overplotting.')
      RETURN
   ENDIF ELSE BEGIN
      IF N_Elements(y) EQ 0 THEN BEGIN
         dep = x
         indep = Lindgen(N_Elements(x))
      ENDIF ELSE BEGIN
         dep = y
         indep = x
      ENDELSE
   ENDELSE

      ; Create an overplot structure and add it to the info structure.

   (*infoptr).overplotconfig->Destroy
   OverplotsNow = (*infoptr).nboverplots + 1
   (*infoptr).nboverplots = OverplotsNow

   IF (*infoptr).nbplotsloaded GE OverplotsNow THEN BEGIN

       OColorArr =  *(*infoptr).OColorArr
       OThickArr =  *(*infoptr).OThickArr
       OLineStyleArr =  *(*infoptr).OLineStyleArr
       OSymSizeArr =  *(*infoptr).OSymSizeArr
       OPSymArr =  *(*infoptr).OPSymArr
       ONoLineArr =  *(*infoptr).ONoLineArr

       struct = {IAA_OVERPLOT, Ptr_New(dep), Ptr_New(indep), Ptr_New(extra),  OColorArr[OverplotsNow-1], $
              OThickArr[OverplotsNow-1], OLineStyleArr[OverplotsNow-1], $
              OSymSizeArr[OverplotsNow-1], OPSymArr[OverplotsNow-1], ONoLineArr[OverplotsNow-1]}

   ENDIF ELSE BEGIN

       noline = 1
       IF psym LT 0 THEN noline=-1
       struct = {IAA_OVERPLOT, Ptr_New(dep), Ptr_New(indep), Ptr_New(extra), color, $
           1., 0, 1., abs(psym), noline}

   ENDELSE

   IF Ptr_Valid((*infoptr).oplotinfo) THEN $
   *(*infoptr).oplotinfo = [*(*infoptr).oplotinfo, struct] ELSE $
   (*infoptr).oplotinfo = Ptr_New([struct])

        ; Draw the plot.

   IAA_Plot_DrawthePlot, infoptr, /Overplot

        ; Allow the clear oplots button to be active.

   Widget_Control, (*infoptr).clearoplotID, Sensitive=1
   Widget_Control, (*infoptr).updoplotID, Sensitive=1

   RETURN
ENDELSE

   ; Sort out the positional parameters for plotting.

IF N_Elements(x) EQ 0 THEN BEGIN
   x = Findgen(101)
   dep = Sin(x/5)/ Exp(x/50)
   Indep = Findgen(101)
ENDIF ELSE BEGIN
   IF N_Elements(y) EQ 0 THEN BEGIN
      dep = x
      indep = Indgen(N_Elements(x))
   ENDIF ELSE BEGIN
      dep = y
      indep = x
   ENDELSE
ENDELSE

   ; Check keywords.

IF N_Elements(title) EQ 0 THEN title=''
IF N_Elements(xtitle) EQ 0 THEN xtitle=''
IF N_Elements(ytitle) EQ 0 THEN ytitle=''
IF N_Elements(xerrplot) EQ 0 THEN xerrplot=''
IF N_Elements(yerrplot) EQ 0 THEN yerrplot=''
IF Size(background, /TName) NE 'STRING' THEN Message, 'BACKGROUND must be specified as color NAME.', /NoName
IF Size(color, /TName) NE 'STRING' THEN Message, 'COLOR must be specified as color NAME.', /NoName
IF Size(datacolor, /TName) NE 'STRING' THEN Message, 'DATACOLOR must be specified as color NAME.', /NoName

   ; Check for availability of GIF files.

thisVersion = Float(!Version.Release)
IF thisVersion LT 5.4 THEN haveGif = 1 ELSE haveGIF = 0

   ; Find the data axes ranges.

mindep = Min(dep, Max=maxdep)
minindep = Min(indep, Max=maxindep)

   ; Create the widgets.

tlb = Widget_Base(Column=1, TLB_Size_Events=1, Base_Align_Center=1, Title='IAA_Plot', KBRD_Focus_Events=1)
drawid = Widget_Draw(tlb, XSize=800, YSize=600, Event_Pro='IAA_Plot_Motion', /motion_events)
buttonbase = Widget_Base(tlb, Row=1)

   ; Quit button.

button = Widget_Button(buttonbase, Value='Quit', Event_Pro='IAA_Plot_Quit', /Dynamic_Resize)

   ; Load button.

button = Widget_Button(buttonbase, Value='Load Settings', Event_Pro='IAA_Plot_Load', /Dynamic_Resize)

   ; Print button.

printID = Widget_Button(buttonbase, Value='Print', Event_Pro='IAA_Plot_Print', /Menu, /Dynamic_Resize)
   dummy = Widget_Button(printID, Value='Landscape Orientation', UVALUE='LANDSCAPE')
   dummy = Widget_Button(printID, Value='Portrait Orientation', UVALUE='PORTRAIT')

   ; Save As button.

saveID = Widget_Button(buttonbase, Value='Save As', Event_Pro='IAA_Plot_SaveAs', /Menu, /Dynamic_Resize)
dummy = Widget_Button(saveID, Value='BMP File', UValue='BMP')
IF havegif THEN dummy = Widget_Button(saveID, Value='GIF File', UValue='GIF')
dummy = Widget_Button(saveID, Value='PICT File', UValue='PICT')
dummy = Widget_Button(saveID, Value='PNG File', UValue='PNG')
dummy = Widget_Button(saveID, Value='JPEG File', UValue='JPG')
dummy = Widget_Button(saveID, Value='TIFF File', UValue='TIF')
psID = Widget_Button(saveID, Value='PostScript File', UValue='PS', $
   Event_Pro='IAA_Plot_Postscript')

   ; Clear overplots.

clearoplotID = Widget_Button(buttonbase, Value='Clear Overplots', $
  Event_Pro='IAA_Plot_Clear_Overplots', /Dynamic_Resize, Sensitive=0)

   ; Modify Overplot button.

updoplotID = Widget_Button(buttonbase, Value='Modify Overplots', $
  Event_Pro='IAA_Plot_Overplots_Button', /Dynamic_Resize, Sensitive=0)

   ; Modify Plot button.

button = Widget_Button(buttonbase, Value='Modify Plot', $
  Event_Pro='IAA_Plot_Configure_Button', /Dynamic_Resize)

   ; Save Settings button.

button = Widget_Button(buttonbase, Value='Save Settings', $
  Event_Pro='IAA_Plot_Save_Settings', /Dynamic_Resize)

  ; About button.

button = Widget_Button(buttonbase, Value='About...', $
  Event_Pro='IAA_Plot_About', /Dynamic_Resize)

   ; Set up a widget to receive the Plot Configuration Events. The
   ; Buttonbase is not being used currently, so is a good choice.

Widget_Control, buttonBase, Event_Pro='IAA_Plot_Configuration_Events'

   ; Print Data
labelbase = Widget_Base(tlb, Row=1)
labelID = widget_label(labelbase, value='  ', /align_center, /dynamic_resize)

   ; Create the axis and plot configuration objects.

xaxis = Obj_New('IAA_Axis', Range=[minindep, maxindep],  /XAxis, _Extra=extra, Title=xtitle, $
   Log=Keyword_Set(xlog))
yaxis = Obj_New('IAA_Axis', Range=[mindep, maxdep], /YAxis, _Extra=extra, Title=ytitle, $
   Log=Keyword_Set(ylog))
plotconfig=Obj_New('IAA_PlotConfig', XAxis=xaxis, YAxis=yaxis, _Extra=extra, $
   Title=title, Color=color, DataColor=datacolor, Background=background, PSym=psym, $
   UseDataColor=1)

   ; Realize the program and make the display window the current graphics window.

Widget_Control, tlb, /Realize, Update=0
Widget_Control, drawid, Get_Value=wid

    ; Get Keywords

plotkeywords = plotconfig->GetKeywords()

   ; Draw the plot.

WSet, wid

Plot, indep, dep, _Extra=plotkeywords, /NoData
plotkeywords.color = plotkeywords.datacolor
OPlot, indep, dep, _Extra=plotkeywords

   ; Draw errplots

IF (N_Elements(xerrplot) GT 1) THEN BEGIN
   oploterr, indep, dep, xerrplot, yerrplot
ENDIF

IF (N_Elements(yerrplot) GT 1) THEN BEGIN
   oploterr, indep, dep, yerrplot
ENDIF

Widget_Control, tlb, Update=1

; Inquire about buttonbase sizes. Needed for resizing events.

geom = Widget_Info(buttonbase, /Geometry)
yminsize = geom.scr_ysize+200
xminsize = geom.scr_xsize

   ; Create an info structure and store it in the UValue of the TLB.

info = {xaxis:xaxis, yaxis:yaxis, wid:wid, dep:dep, indep:indep, plotconfig:plotconfig, overplotconfig:overplotconfig, $
   nboverplots:nboverplots, buttonbase:buttonbase, drawid:drawid, yminsize:yminsize, xminsize:xminsize, oplotinfo:Ptr_New(), $
   x:!X, y:!Y, p:!p, clearoplotID:clearoplotID, updoplotID:updoplotID, labelID:labelID, xerrplot:xerrplot, yerrplot:yerrplot, $
   OColorArr:Ptr_New(), OThickArr:Ptr_New(), OLineStyleArr:Ptr_New(), OSymSizeArr:Ptr_New(), OPSymArr:Ptr_New(), ONoLineArr:Ptr_New(), $
   nbplotsloaded:nbplotsloaded}

infoPtr = Ptr_New(info, /No_Copy)
Widget_Control, tlb, set_uvalue=infoPtr

XManager, 'iaa_plot', tlb, /No_Block, Cleanup='IAA_Plot_Cleanup', $
   Event_Handler='IAA_Plot_Resize', Group_Leader=group_leader
END
